#!/usr/bin/env python

#  $Id: tester2.py 978 2012-06-26 03:12:12Z jemian $


"""
test various regular expressions to locate arguments 
to cdef for a particular case from standard.mac
"""

import re


s = u'"cdef_macro", sprintf("dscan_cleanup $1 %s;", _c1), "cdef_part", flags'

WORD_STRING_MATCH = '([a-z_]\w*)'
DOUBLE_QUOTE_STRING_MATCH = '(".*?")'
DELIMITER_STRING_MATCH = ",?\s*"
WORD_OR_DQS_MATCH = DOUBLE_QUOTE_STRING_MATCH+'|'+WORD_STRING_MATCH
WORD_OR_DQS_MATCH = '([a-z_]\w*|".*?")'

arg1_re = re.compile(DOUBLE_QUOTE_STRING_MATCH, re.IGNORECASE|re.DOTALL)
arg2_re = re.compile(DOUBLE_QUOTE_STRING_MATCH 
                     + DELIMITER_STRING_MATCH 
                     + DOUBLE_QUOTE_STRING_MATCH
                     , re.IGNORECASE|re.DOTALL)
arg3_re = re.compile(DOUBLE_QUOTE_STRING_MATCH 
                     + DELIMITER_STRING_MATCH 
                     + DOUBLE_QUOTE_STRING_MATCH
                     + DELIMITER_STRING_MATCH 
                     + DOUBLE_QUOTE_STRING_MATCH
                     , re.IGNORECASE|re.DOTALL)
arg4_re = re.compile(DOUBLE_QUOTE_STRING_MATCH 
                     + DELIMITER_STRING_MATCH 
                     + WORD_OR_DQS_MATCH
                     + DELIMITER_STRING_MATCH 
                     + DOUBLE_QUOTE_STRING_MATCH
                     + DELIMITER_STRING_MATCH 
                     + WORD_OR_DQS_MATCH
                     , re.IGNORECASE|re.DOTALL)

for i, regexp in enumerate((arg1_re, arg2_re, arg3_re, arg4_re)):
    m = regexp.match(s)
    if m is not None:
        print i, m.groups()



# -*- coding: utf-8 -*-
"""
    sphinxcontrib.specdomain
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    :synopsis: SPEC domain for Sphinx
    
    Automatically insert ReST-formatted extended comments 
    from SPEC files for macro definitions and variable declarations into
    the Sphinx doctree, thus avoiding duplication between docstrings and documentation
    for those who like elaborate docstrings.

    :copyright: Copyright 2012-2014 by BCDA, Advanced Photon Source, Argonne National Laboratory
    :license: ANL Open Source License, see LICENSE for details.
"""

# http://sphinx.pocoo.org/ext/appapi.html

import os
import re

from docutils import nodes

from sphinx import addnodes
from sphinx.roles import XRefRole
from sphinx.locale import l_, _
from sphinx.directives import ObjectDescription
from sphinx.domains import Domain, ObjType
from sphinx.util.nodes import make_refnode
from sphinx.util.docfields import Field, TypedField
from sphinx.util.docstrings import prepare_docstring

from sphinx.ext.autodoc import Documenter, bool_option
from specmacrofileparser import SpecMacrofileParser


# TODO: merge these with specmacrofileparser.py
match_all                   = r'.*'
non_greedy_filler           = match_all + r'?'
double_quote_string_match   = r'("' + non_greedy_filler + r'")'
word_match                  = r'((?:[a-z_]\w*))'
cdef_match                  = r'(cdef)'


spec_macro_sig_re = re.compile(
                               r'''^ ([a-zA-Z_]\w*)         # macro name
                               ''', re.VERBOSE)

spec_func_sig_re = re.compile(word_match + r'\('
                      + r'(' + match_all + r')' 
                      + r'\)', 
                      re.IGNORECASE|re.DOTALL)

spec_cdef_name_sig_re = re.compile(double_quote_string_match, 
                                   re.IGNORECASE|re.DOTALL)

# this tool is valuable:  http://www.pythonregex.com/
spec_macro_file_re_str = "\w*.mac"  # TRAC #29: can user provide somehow (can't get to it from here if defined in conf.py)?
spec_macro_file_re = re.compile(spec_macro_file_re_str)


def isSpecMacroFile(filename):
    '''is filename a SPEC macro file?'''
    return os.path.isfile(filename) and len(spec_macro_file_re.findall(filename)) > 0


class SpecMacroDocumenter(Documenter):
    """
    Document a SPEC macro source code file (autodoc.Documenter subclass)
    
    This code responds to the ReST file directive::
    
        .. autospecmacro:: partial/path/name/somefile.mac
            :displayorder: fileorder
    
    The ``:displayorder`` parameter indicates how the
    contents will be sorted for appearance in the ReST document.
    
        **fileorder** or **file**
            Items will be documented in the order in 
            which they appear in the ``.mac`` file.
            (Default)
        
        **alphabetical** or **alpha**
            Items will be documented in alphabetical order.
            (Not implemented at present.)
    
    .. tip::
        A (near) future enhancement will provide for
        documenting all macro files in a directory, with optional
        recursion into subdirectories.  By default, the code will 
        only document files that match the glob pattern ``*.mac``.
        (This could be defined as a list in the ``conf.py`` file.)
        Such as::
        
           .. spec:directory:: partial/path/name
              :recursion:
              :displayorder: alphabetical
    """

    objtype = 'specmacro'
    member_order = 50
    priority = 0
    #: true if the generated content may contain titles
    titles_allowed = True

    option_spec = {
        'displayorder': bool_option,
    }

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        # don't document submodules automatically
        #return isinstance(member, (FunctionType, BuiltinFunctionType))
        r = membername in ('SpecMacroDocumenter', )
        return r
    
    def generate(self, *args, **kw):
        """
        Generate reST for the object given by *self.name*, and possibly for
        its members.
        """
        # now, parse the SPEC macro file
        macrofile = self.parse_name()
        
        # <hack>
        # SPEC macro files are given by paths relative to documentation tree
        # What about absolute paths?  How does Sphinx handle that?
        # macro directory (with the mac file)
        #mdir = os.path.dirname(os.path.abspath(macrofile))
        # source directory (with the rst file)
        sdir = os.path.abspath(os.path.join(self.directive.env.srcdir, self.directive.env.docname))
        # present directory (where the Makefile started this)
        pdir = os.path.abspath(os.getcwd())
        # get the relative path from sdir to pdir (assumes sdir is below pdir)
        # minus one for the .rst doc at the end
        dir_levels = len(sdir.split(os.sep))-len(pdir.split(os.sep))-1
        macrofile_prefix = '../'*dir_levels
        # </hack>

        spec = SpecMacrofileParser(macrofile)
        extended_comment = spec.ReST()
        rest = prepare_docstring(extended_comment)

        #self.add_line(u'', '<autodoc>')
        #sig = self.format_signature()
        #self.add_directive_header(sig)
        
        self.add_line(u'', '<autodoc>')
        self.add_line(u'.. index:: SPEC macro file; %s' % macrofile, '<autodoc>')
        self.add_line(u'.. index:: !%s' % os.path.split(macrofile)[1], '<autodoc>')
        self.add_line(u'', '<autodoc>')
        self.add_line(u'', '<autodoc>')
        title = 'SPEC Macro File: %s' %  macrofile
        self.add_line('@'*len(title), '<autodoc>')
        self.add_line(title, '<autodoc>')
        self.add_line('@'*len(title), '<autodoc>')
        self.add_line(u'', '<autodoc>')
        # TODO: provide links from each to highlighted source code blocks (like Python documenters).
        # This will have to do for now.
        line = 'source code:  :download:`%s <%s>`' % (os.path.basename(macrofile), macrofile_prefix+macrofile)
        self.add_line(line, macrofile)

        self.add_line(u'', '<autodoc>')
        for linenumber, line in enumerate(rest):
            self.add_line(line, macrofile, linenumber)

        #self.add_content(rest)
        #self.document_members(all_members)

    def resolve_name(self, modname, parents, path, base):
        if modname is not None:
            self.directive.warn('"::" in autospecmacro name doesn\'t make sense')
        return (path or '') + base, []

    def parse_name(self):
        """Determine what file to parse.
        
        :returns: True if if parsing was successful
        """
        ret = self.name
        self.fullname = os.path.abspath(ret)
        self.objpath, self.modname = os.path.split(self.fullname)
        self.args = None
        self.retann = None
        if self.args or self.retann:
            self.directive.warn('signature arguments or return annotation '
                                'given for autospecmacro %s' % self.fullname)
        return ret


class SpecDirDocumenter(Documenter):
    """
    Document a directory containing SPEC macro source code files.
    
    This code responds to the ReST file directive::
    
        .. autospecdir:: partial/path/name
    """
    objtype = 'specdir'
    member_order = 50
    priority = 0
    #: true if the generated content may contain titles
    titles_allowed = True
    option_spec = {
        'include_subdirs': bool_option,
    }

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return membername in ('SpecDirDocumenter', )
    
    def generate(self, *args, **kw):
        """
        Look at the named directory and generate reST for the 
        object given by *self.name*, and possibly for its members.
        """
        # now, parse the .mac files in the SPEC directory
        specdir = self.name
#        self.add_line(u'', '<autodoc>')
#        self.add_line(u'directory:\n   ``%s``' % specdir, '<autodoc>')
        macrofiles = []
        if os.path.exists(specdir):
            # TODO: support a user choice for pattern match to the file name (glob v. re)
            # see https://subversion.xray.aps.anl.gov/trac/bcdaext/ticket/29
            for f in sorted(os.listdir(specdir)):
                filename = os.path.join(specdir, f)
                if isSpecMacroFile(filename):
                    # TODO: support the option to include subdirectories (include_subdirs)
                    # TODO: do not add the same SPEC macro file more than once
                    macrofiles.append(filename)
        else:
            self.add_line(u'', '<autodoc>')
            self.add_line(u'Could not find directory: ``%s``' % specdir, '<autodoc>')
        if len(macrofiles) > 0:
            self.add_line(u'', '<autodoc>')
            self.add_line(u'.. rubric:: List of SPEC Macro Files in *%s*' % specdir, '<autodoc>')
            self.add_line(u'', '<autodoc>')
            for filename in macrofiles:
                # Show a bullet list at the top of the page
                # This is an alternative to separate pages for each macro file
                self.add_line(u'* :ref:`%s <%s>`' % (filename, filename), '<autodoc>')
            self.add_line(u'', '<autodoc>')
            self.add_line(u'-'*15, '<autodoc>')         # delimiter
            self.add_line(u'', '<autodoc>')
            for filename in macrofiles:
                self.add_line(u'', '<autodoc>')
                self.add_line(u'.. _%s:' % filename, '<autodoc>')
                self.add_line(u'.. autospecmacro:: %s' % filename, '<autodoc>')
                # TODO: any options?
                self.add_line(u'', '<autodoc>')
                # TODO: suppress delimiter after last file
                self.add_line(u'-'*15, '<autodoc>')         # delimiter between files


class SpecMacroObject(ObjectDescription):
    """
    Description of a SPEC macro definition
    """

    doc_field_types = [
        TypedField('parameter', label=l_('Parameters'),
                   names=('param', 'parameter', 'arg', 'argument',
                          'keyword', 'kwarg', 'kwparam'),
                   typerolename='def', typenames=('paramtype', 'type'),
                   can_collapse=True),
        Field('returnvalue', label=l_('Returns'), has_arg=False,
              names=('returns', 'return')),
        Field('returntype', label=l_('Return type'), has_arg=False,
              names=('rtype',)),
    ]

    def add_target_and_index(self, name, sig, signode):
        targetname = 'macro:%s:%s:%s:%s' % (self.objtype, name, signode.source, str(signode.line))
        signode['ids'].append(targetname)
        self.state.document.note_explicit_target(signode)
        indextext = self._get_index_text(name)
        if indextext:
            self.indexnode['entries'].append(('single', indextext, targetname, ''))
            self.indexnode['entries'].append(('single', sig, targetname, ''))
            # TODO: what if there is more than one file, same name, different path?
            filename = os.path.split(signode.document.current_source)[1]
            if isSpecMacroFile(filename):
                indextext = '%s; %s' % (filename, sig)
                self.indexnode['entries'].append(('single', indextext, targetname, ''))

    macro_types = {
        'def':  'SPEC macro definition; %s',
        'rdef': 'SPEC run-time macro definition; %s',
        'cdef': 'SPEC chained macro definition; %s',
    }

    def _get_index_text(self, name):
        if self.objtype in self.macro_types:
            return _(self.macro_types[self.objtype]) % name
        else:
            return ''

    def handle_signature(self, sig, signode):
        '''return the name of this object from its signature'''
        # Must be able to match these (without preceding def or rdef)
        #     def macro_name
        #     def macro_name()
        #     def macro_name(arg1, arg2)
        #     rdef macro_name
        #     cdef("macro_name", "content", "groupname", flags)
        m = spec_func_sig_re.match(sig) or spec_macro_sig_re.match(sig)
        if m is None:
            raise ValueError
        arglist = m.groups()
        name = arglist[0]
        args = []
        if len(arglist) > 1:
            args = arglist[1:]
            if name == 'cdef':
                # TODO: need to match complete arg list
                # several different signatures are possible (see cdef-examples.mac)
                # for now, just get the macro name and ignore the arg list
                m = spec_cdef_name_sig_re.match(args[0])
                arglist = m.groups()
                name = arglist[0].strip('"')
                args = ['<<< cdef argument list not handled yet >>>']       # FIXME:
        signode += addnodes.desc_name(name, name)
        if len(args) > 0:
            signode += addnodes.desc_addname(args, args)
        return name


class SpecVariableObject(ObjectDescription):
    """
    Description of a SPEC variable
    """
    
    # TODO: The directive that declares the variable should be the primary (bold) index.
    # TODO: array variables are not handled at all
    # TODO: variables cited by *role* should link back to their *directive* declarations

    def handle_signature(self, sig, signode):
        '''return the name of this object from its signature'''
        # TODO: Should it match a regular expression?
        # TODO: What if global or local?  
        signode += addnodes.desc_name(sig, sig)
        return sig

    def add_target_and_index(self, name, sig, signode):
        #text = u'! ' + sig      # TODO: How to use emphasized index entry in this context?
        text = name.split()[0]   # when sig = "tth    #: scattering angle"
        targetname = 'var:%s:%s:%s:%s' % (self.objtype, text, signode.source, str(signode.line))
        signode['ids'].append(targetname)
        # TODO: role does not point back to it yet
        # http://sphinx.pocoo.org/markup/misc.html#directive-index
        self.indexnode['entries'].append(('single', text, targetname, ''))
        text = u'SPEC %s variable; %s' % (self.objtype, sig)
        self.indexnode['entries'].append(('single', text, targetname, ''))

class SpecXRefRole(XRefRole):
    """ Cross-reference the roles in specdomain """
    
    def process_link(self, env, refnode, has_explicit_title, title, target):
        """Called after parsing title and target text, and creating the
        reference node (given in *refnode*).  This method can alter the
        reference node and must return a new (or the same) ``(title, target)``
        tuple.
        """
        key = ":".join((refnode['refdomain'], refnode['reftype']))
        value = env.temp_data.get(key)
        refnode[key] = value
        if not has_explicit_title:
            title = title.lstrip(':')   # only has a meaning for the target
            target = target.lstrip('~') # only has a meaning for the title
            # if the first character is a tilde, don't display the module/class
            # parts of the contents
            if title[0:1] == '~':
                title = title[1:]
                colon = title.rfind(':')
                if colon != -1:
                    title = title[colon+1:]
        return title, target

    def result_nodes(self, document, env, node, is_ref):
        """Called before returning the finished nodes.  *node* is the reference
        node if one was created (*is_ref* is then true), else the content node.
        This method can add other nodes and must return a ``(nodes, messages)``
        tuple (the usual return value of a role function).
        """
        # this code adds index entries for each role instance
        if not is_ref:
            return [node], []
        varname = node['reftarget']
        tgtid = 'index-%s' % env.new_serialno('index')
        indexnode = addnodes.index()
        indexnode['entries'] = [
            ('single', varname, tgtid, ''),
            #('single', _('environment variable; %s') % varname, tgtid, ''),
        ]
        targetnode = nodes.target(node.rawsource, '', ids=[tgtid])
        document.note_explicit_target(targetnode)
        return [indexnode, targetnode, node], []


class SpecDomain(Domain):
    """SPEC language domain."""
    
    name = 'spec'
    label = 'SPEC, http://www.certif.com'
    object_types = {    # type of object that a domain can document
        'def':        ObjType(l_('def'),        'def'),
        'rdef':       ObjType(l_('rdef'),       'rdef'),
        'cdef':       ObjType(l_('cdef'),       'cdef'),
        'global':     ObjType(l_('global'),     'global'),
        'local':      ObjType(l_('local'),      'local'),
        'constant':   ObjType(l_('constant'),   'constant'),
        #'specmacro':  ObjType(l_('specmacro'),  'specmacro'),
    }
    directives = {
        'def':          SpecMacroObject,
        'rdef':         SpecMacroObject,
        'cdef':         SpecMacroObject,
        'global':       SpecVariableObject,
        'local':        SpecVariableObject,
        'constant':     SpecVariableObject,
    }
    roles = {
        'def' :     SpecXRefRole(),
        'rdef':     SpecXRefRole(),
        'cdef':     SpecXRefRole(),
        'global':   SpecXRefRole(),
        'local':    SpecXRefRole(),
        'constant': SpecXRefRole(),
    }
    initial_data = {
        'objects': {}, # fullname -> docname, objtype
    }

    def clear_doc(self, docname):
        for (typ, name), doc in self.data['objects'].items():
            if doc == docname:
                del self.data['objects'][typ, name]

    def resolve_xref(self, env, fromdocname, builder, typ, target, node,
                     contnode):
        objects = self.data['objects']
        objtypes = self.objtypes_for_role(typ)
        for objtype in objtypes:
            if (objtype, target) in objects:
                return make_refnode(builder, fromdocname,
                                    objects[objtype, target],
                                    objtype + '-' + target,
                                    contnode, target + ' ' + objtype)

    def get_objects(self):
        for (typ, name), docname in self.data['objects'].iteritems():
            yield name, name, typ, docname, typ + '-' + name, 1


# http://sphinx.pocoo.org/ext/tutorial.html#the-setup-function

def setup(app):
    app.add_domain(SpecDomain)
    app.add_autodocumenter(SpecMacroDocumenter)
    app.add_autodocumenter(SpecDirDocumenter)
    app.add_config_value('autospecmacrodir_process_subdirs', True, True)



#!/usr/bin/env python


'''
$Id: tester4.py 983 2012-06-28 21:34:54Z jemian $

develop regular expression to match macro declarations
'''


import re
import os

# TODO: consider multiline declaration signtures
# This test module only considers single line declarations

spec_macro_declaration_match_re = re.compile(
                              r'^'                      # line start
                            + r'\s*?'                   # optional blank space
                            + r'(r?def)'                # 0: def_type (rdef | def)
                            + r'\s*?'                   # optional blank space
                            + r'([a-zA-Z_][\w_]*)'      # 1: macro_name
                            + r'(.*?)'                  # 2: optional arguments
                            + r'(#.*?)?'                # 3: optional comment
                            + r'$'                      # line end
                        )

spec_cdef_declaration_match_re = re.compile(
                              r'^'                      # line start
                            + r'.*?'                    # optional any kind of preceding stuff, was \s*? (optional blank space)
                            + r'(cdef)'                 # 0: cdef
                            + r'\('                     # opening parenthesis
                            + r'(.*?)'                  # 1: args (anything between the parentheses)
                            + r'\)'                     # closing parenthesis
                            + r'.*?'                    # optional any kind of stuff
                            + r'(#.*?)?'                # 2: optional comment with content
                            + r'$'                      # line end
                        )

spec_function_declaration_match_re = re.compile(
                              r'^'                      # line start
                            + r'\s*?'                   # optional blank space
                            + r'(r?def)'                # 0: def_type (rdef | def)
                            + r'\s*?'                   # optional blank space
                            + r'([a-zA-Z_][\w_]*)'      # 1: function_name
                            + r'\('                     # opening parenthesis
                            + r'(.*?)'                  # 2: args (anything between the parentheses)
                            + r'\)'                     # closing parenthesis
                            + r'\s*?'                   # optional blank space
                            + r'\''                     # open macro content
                            + r'(.*?)'                  # 3: args (anything between the parentheses)
                            + r'(#.*?)?'                # 4: optional comment
                            + r'$'                      # line end
                        )

TEST_DIR = os.path.join('..', 'macros')

tests = {
    'macro':    spec_macro_declaration_match_re,
    'cdef':     spec_cdef_declaration_match_re,
    'function': spec_function_declaration_match_re,
}

for name, test in tests.items():
    print "%10s  %s" % (name, test.pattern)

for f in sorted(os.listdir(TEST_DIR)):
    if f.endswith('.mac'):
        print '\n', f
        for linenumber, line in enumerate(open(os.path.join(TEST_DIR, f)).readlines()):
            for name in ('cdef', 'function', 'macro'):
                m = tests[name].match(line)
                if m is not None:
                    print name, linenumber+1, m.groups()
                    break



#!/usr/bin/env python

########### SVN repository information ###################
# $Date: 2014-03-11 01:05:15 -0500 (Tue, 11 Mar 2014) $
# $Author: jemian $
# $Revision: 1515 $
# $HeadURL: https://subversion.xray.aps.anl.gov/bcdaext/specdomain/trunk/src/specdomain/sphinxcontrib/specmacrofileparser.py $
# $Id: specmacrofileparser.py 1515 2014-03-11 06:05:15Z jemian $
########### SVN repository information ###################


"""
Construct a SPEC macro source code file parser for 
use by the specdomain for Sphinx.  This parser locates
code blocks in the SPEC macro source code file across multiple lines.

:copyright: Copyright 2012-2014 by BCDA, Advanced Photon Source, Argonne National Laboratory
:license: ANL Open Source License, see LICENSE for details.
"""

import os
import re
from pprint import pprint        #@UnusedImport

#   http://www.txt2re.com/index-python.php3
#  http://regexpal.com/

string_start                = r'^'
string_end                  = r'$'
match_all                   = r'.*'
non_greedy_filler           = match_all + r'?'
non_greedy_whitespace       = r'\s*?'
#double_quote_string_match   = r'("' + non_greedy_filler + r'")'
#prog_name_match             = r'([a-z_]\w*)'
#word_match                  = r'((?:[a-z_]\w*))'
#cdef_match                  = r'(cdef)'
extended_comment_marker     = r'\"{3}'
extended_comment_match      = r'(' + extended_comment_marker + r')'
macro_name                  = r'[a-zA-Z_]\w*'
macro_name_match            = r'(' + macro_name + r')'
arglist_match               = r'(' + match_all + r')'
non_greedy_filler_match     = r'(' + non_greedy_filler + r')'
variable_name_match         = r'(@?' + macro_name + r'\[?\]?)'


        
extended_comment_block_sig_re = re.compile(
                        string_start
                        + non_greedy_whitespace
                        + extended_comment_marker
                        + r'(' + non_greedy_filler + r')'
                        + extended_comment_marker
                        + non_greedy_filler
                        + string_end, 
                        re.IGNORECASE|re.DOTALL|re.MULTILINE)

variable_description_re = re.compile(
                        string_start
                        + non_greedy_filler
                        + r'#:'
                        + non_greedy_whitespace
                        + r'(' + non_greedy_filler + r')'
                        + non_greedy_whitespace
                        + string_end, 
                        re.IGNORECASE|re.DOTALL|re.MULTILINE)

    
lgc_variable_sig_re = re.compile(
                        r''
                        + string_start
                        + non_greedy_whitespace
                        + r'(local|global|constant)'        # 1: object type
                        + non_greedy_whitespace
                        + r'(' + non_greedy_filler + r')'   # 2: too complicated to parse all at once
                        + string_end
                        , 
                        re.DOTALL
                        |re.MULTILINE
                    )

variable_name_re = re.compile(
                        variable_name_match, 
                        re.IGNORECASE|re.DOTALL|re.MULTILINE
                        )

arg_list_match = r'(\(' + non_greedy_filler + r'\))?'

spec_macro_declaration_match_re = re.compile(
                        string_start
                        + r'\s*?'                           # optional blank space
                        + r'(r?def)\s'                      # 1: def_type (rdef | def)
                        + non_greedy_whitespace
                        + macro_name_match                  # 2: macro_name
                        + arg_list_match                    # 3: optional arguments
                        + non_greedy_whitespace
                        + r"[\\']+"                          # start body section
                        + non_greedy_filler_match           # 4: body
                        + r"[\\']+"                          # end body section
                        + non_greedy_whitespace
                        + r'(#.*?)?'                        # 5: optional comment
                        + string_end
                        , 
                        re.IGNORECASE|re.DOTALL|re.MULTILINE)



args_match_re = re.compile(
                          r'\('
                        + arglist_match                     # 1:  argument list
                        + r'\)', 
                        re.DOTALL)

class SpecMacrofileParser:
    '''
    Parse a SPEC macro file for macro definitions, 
    variable declarations, and extended comments.

        Since 2002, SPEC has allowed for triple-quoted 
        strings as extended comments.  Few, if any, have used them.
        Assume all extended comments contain ReST formatted comments, 
        *including initial section titles or transitions*.
    
    Assume macro definitions are not nested (but test for this).
    
    Assume macro files are small enough to load completely in memory.
        
    An additional step would be to parse for:
    * def
    * cdef
    * rdef
    * global
    * local
    * constant
    * array
    * ...
    '''
    
    def __init__(self, macrofile):
        self.buf = None
        self.findings = []
        self.filename = None
        self.read(macrofile)
        self.parse_macro_file()
        self.description = ''
        self.clear_description = False
        self.found_first_global_extended_comment = False
    
    def read(self, macrofile):
        """
        load the SPEC macro source code file into an internal buffer (self.buf).
        Also remember the start and end position of each line (self.line_positions).
        
        :param str filename: name (with optional path) of SPEC macro file
            (The path is relative to the ``.rst`` document.)
        """
        if not os.path.exists(macrofile):
            raise RuntimeError, "file not found: " + macrofile
        self.filename = macrofile
        buf = open(macrofile, 'r').readlines()
        offset = 0
        lines = []
        for linenumber, line in enumerate(buf):
            end = offset+len(line)
            lines.append([linenumber+1, offset, end])
            offset = end
        self.buf = ''.join(buf)
        self.line_positions = lines
    
    def std_read(self, macrofile):
        """
        load the SPEC macro source code file into an internal buffer
        
        :param str filename: name (with optional path) of SPEC macro file
            (The path is relative to the ``.rst`` document.)
        """
        if not os.path.exists(macrofile):
            raise RuntimeError, "file not found: " + macrofile
        self.filename = macrofile
        self.buf = open(macrofile, 'r').read()

    def parse_macro_file(self):
        """
        Figure out what can be documented in the file's contents (in self.buf)
        
            each of the list_something() methods returns a 
            list of dictionaries where each dictionary 
            has the keys: objtype, start_line, end_line, and others
        """
        db = self._make_db()        # first, the file parsing
        
        # Build a dict with objecttype for keys and methods for values
        # each method handles that particular spec macro file structure
        handler_method = {
            'cdef': self.handle_other,
            'constant': self.handle_other,
            'def': self.handle_def,
            'descriptive comment': self.handle_descriptive_comment,
            'extended comment': self.handle_extended_comment,
            'function def': self.handle_def,
            'function rdef': self.handle_def,
            'global': self.handle_other,
            'local': self.handle_other,
            'rdef': self.handle_def,
        }
        process_first_list = ('descriptive comment', )
        
        # then analyze what was found
        # proceed line-by-line in order
        # TODO: could override this rule with a sort-order option
        self.findings = []
        self.description = ''
        self.clear_description = False
        self.found_first_global_extended_comment = False
        for linenumber in sorted(db.keys()):
            # Diagnostic line for development only
            #print linenumber, ':', ' '.join(['<%s>' % d['objtype'] for d in db[linenumber]])
            
            # process any descriptive comment first
            for item in db[linenumber]:
                if item['objtype'] in process_first_list:
                    handler_method[item['objtype']](item, db)
            # now process the others
            for item in db[linenumber]:
                if item['objtype'] not in process_first_list:
                    if 'function rdef' == item['objtype']:
                        pass
                    handler_method[item['objtype']](item, db)
            
            if self.clear_description:
                self.description, self.clear_description = '', False
    
    def _make_db(self):
        """build the db dict by parsing for each type of structure"""
        db = {}
        # first, the file parsing
        for func in (self.list_def_macros, 
                     self.list_cdef_macros,
                     self.list_variables,
                     self.list_extended_comments,
                     self.list_descriptive_comments,
                     ):
            for item in func():
                s = item['start_line']
                if s not in db.keys():
                    db[s] = []
                db[s].append(item)
        return db

    def handle_def(self, node, db):
        """document SPEC def structures"""
        # identify all the children of this node
        parent = node['name']
        self.found_first_local_extended_comment = False
        if node.get('comment') is not None:
            node['description'] = node.get('comment').lstrip('#:').strip()
        if len(self.description)>0:
            node['description'] = self.description
        for row in xrange(node['start_line']+1, node['end_line']-1):
            if row in db.keys():
                for item in db[row]:
                    item['parent'] = parent
                    if item['objtype'] == 'extended comment':
                        if not self.found_first_local_extended_comment:
                            # TODO: could override this rule with an option
                            node['description'] = item['text']
                            self.found_first_local_extended_comment = False
        if not node['name'].startswith('_'):
            # TODO: could override this rule with an option
            self.findings.append(node)
        node['summary'] = self._extract_summary(node.get('description', ''))
        self.clear_description = True
    
    def handle_descriptive_comment(self, node, db):
        """document SPEC descriptive comment structures"""
        self.description = node['text']
    
    def handle_extended_comment(self, node, db):
        """document SPEC extended comment structures"""
        #start = node['start_line']
        if node['parent'] == None:
            if not self.found_first_global_extended_comment:
                # TODO: could override this rule with an option
                self.findings.append(node)
                self.found_first_global_extended_comment = True
    
    def handle_other(self, node, db):
        """document SPEC cdef, constant, global, local, and rdef structures"""
        if len(self.description)>0:
            node['description'] = self.description
            node['summary'] = self._extract_summary(self.description)
            self.clear_description = True
        if not node['name'].startswith('_'):
            # TODO: could override this rule with an option
            self.findings.append(node)
    
#    def _handle_ignore(self, node, db):
#        """call this handler to ignore an identified SPEC macro file structure"""
#        pass

    def _extract_summary(self, description):
        """
        return the short summary line from the item description text
        
        The summary line is the first line in the docstring,
        such as the line above.
        
        For our purposes now, we return the first paragraph, 
        if it is not a parameter block such as ``:param var: ...``.
        """
        if len(description) == 0:
            return ''
        text = []
        for line in description.strip().splitlines():
            if len(line.strip()) == 0:
                break
            if not line.strip().startswith(':'):
                text.append(line)
        return ' '.join(text)

    def list_extended_comments(self):
        """
        parse the internal buffer for triple-quoted strings, possibly multiline
        
        Usually, an extended comment is used at the top of a macro file
        to describe the file's contents.  It is also used at the top 
        of a macro definition to describe the macro.  The first line
        of an extended comment for a macro should be a short summary,
        followed by a blank line, then either a parameter list or
        more extensive documentation, as needed.
        """
        items = []
        for mo in extended_comment_block_sig_re.finditer(self.buf):
            start = self.find_pos_in_line_number(mo.start(1))
            end = self.find_pos_in_line_number(mo.end(1))
            text = mo.group(1)
            items.append({
                            'start_line': start, 
                            'end_line':   end, 
                            'objtype':    'extended comment',
                            'text':       text,
                            'parent':     None,
                          })
        return items

    def list_descriptive_comments(self):
        """
        Descriptive comments are used to document items that cannot contain
        extended comments (triple-quoted strings) such as variable declarations
        or *rdef* or *cdef* macro declarations.  They appear either in-line
        with the declaration or on the preceding line.
        
        Descriptive comment example that documents *tth*, a global variable declaration::
            
            global tth    #: two-theta, the scattering angle
        
        Descriptive comment example that documents *ccdset_shutter*, a *rdef* declaration::
        
            #: clear the ccd shutter handler
            rdef ccdset_shutter ''
        """
        items = []
        for mo in variable_description_re.finditer(self.buf):
            start = self.find_pos_in_line_number(mo.start(1))
            end = self.find_pos_in_line_number(mo.end(1))
            items.append({
                            'start_line': start, 
                            'end_line':   end, 
                            'objtype':    'descriptive comment',
                            'text':       mo.group(1),
                            'parent':     None,
                          })
        return items

    def list_variables(self):
        """
        parse the internal buffer for local, global, and constant variable declarations
        """
        items = []
        for mo in lgc_variable_sig_re.finditer(self.buf):
            start = self.find_pos_in_line_number(mo.start(1))
            end = self.find_pos_in_line_number(mo.end(1))
            objtype = mo.group(1)
            content = mo.group(2)
            p = content.find('#')
            if p >= 0:                                      # strip off any comment
                content = content[:p]
            content = re.sub('[,;]', ' ', content)          # replace , or ; with blank space
            if content.find('[') >= 0:
                content = re.sub('\s*?\[', '[', content)    # remove blank space before [
            if objtype in ('constant'):
                name = content.strip().split()[0]
                items.append({
                                'start_line': start, 
                                'end_line':   end, 
                                'objtype':    objtype,
                                'name':       name,
                                'parent':     None,
                              })
            else:
                for var in variable_name_re.finditer(content):
                    name = var.group(1)
                    if len(name) > 0:
                        items.append({
                                        'start_line': start, 
                                        'end_line':   end, 
                                        'objtype':    objtype,
                                        'name':       name,
                                        'parent':     None,
                                      })
        return items

    def list_def_macros(self):
        """
        parse the internal buffer for def and rdef macro declarations
        """
        items = []
        for mo in spec_macro_declaration_match_re.finditer(self.buf):
            objtype = mo.group(1)
            start = self.find_pos_in_line_number(mo.start(1))
            end = self.find_pos_in_line_number(mo.end(4))
            args = mo.group(3)
            # TODO: What if args is multi-line?  flatten.  What if really long?
            if start == 225 and end == 225:
                pass
            if args is not None:
                if len(args)>2:
                    m = args_match_re.search(args)
                    if m is not None:
                        objtype = 'function ' + objtype
                        if 'function rdef' == objtype:
                            pass  # TODO: Should we do something special here?
                        args = m.group(1)
                elif args == '()':
                    objtype = 'function ' + objtype
            d = {
                'start_line': start, 
                'end_line':   end, 
                'objtype':    objtype,
                'name':       mo.group(2),
                'args':       str(args),
                'body':       mo.group(4),
                'comment':    mo.group(5),
                'parent':     None,
            }
            items.append(d)
        return items

    def list_cdef_macros(self):
        """
        parse the internal buffer for cdef macro declarations
        """
        # too complicated for a regular expression, just look for the initial part
        items = []
        for mo in re.finditer('cdef\s*?\(', self.buf):
            # look at each potential cdef declaration
            objtype = 'cdef'
            start = self.find_pos_in_line_number(mo.start())
            s = p = mo.end()                # offset s for start of args
            nesting = 1                     # number of nested parentheses
            sign = {'(': 1, ')': -1}        # increment or decrement
            while nesting > 0 and p < len(self.buf):
                if self.buf[p] in sign.keys():
                    nesting += sign[self.buf[p]]
                p += 1
            e = p
            text = self.buf[s:e-1]    # carve it out, and remove cdef( ... ) wrapping
            end = self.find_pos_in_line_number(e)
            p = text.find(',')
            name = text[:p].strip('"')
            if len(name) == 0:
                name = '<empty name>'
            args = text[p+1:]
            # TODO: parse "args" for content
            # TODO: What if args is multi-line?  convert \n to ;
            #   args = ';'.join(args.splitlines())  # WRONG: This converts string content, as well
            # TODO: What if args is really long?
            items.append({
                            'start_line': start, 
                            'end_line':   end, 
                            'objtype':    objtype,
                            'name':       name,
                            'args':       args,
#                            'body':       mo.group(4),
#                            'comment':    mo.group(5),
                            'parent':     None,
                          })
        return items

    def find_pos_in_line_number(self, pos):
        """
        find the line number that includes *pos*
        
        :param int pos: position in the file
        """
        # TODO: optimize this straight search using a search by bisection
        linenumber = None
        for linenumber, start, end in self.line_positions:
            if start <= pos < end:
                break
        return linenumber
    
    #------------------------ reporting section below ----------------------------------

    def _simple_ReST_renderer(self):
        """create a simple ReStructured Text rendition of the findings"""
        declarations = []       # variables and constants
        macros = []             # def, cdef, and rdef macros
        functions = []          # def and rdef function macros
        s = []
        for r in self.findings:
            # TODO: need to define subsections such as these:
            #    Summary (if present)
            #    Documentation (if present)
            #    Declarations
            #    Tables
            if r['objtype'] == 'extended comment':
                # TODO: apply rules to suppress reporting under certain circumstances
                s.append( '' )
                s.append( '.. %s %s %d %d' % (self.filename, 
                                              r['objtype'], 
                                              r['start_line'], 
                                              r['end_line']) )
                s.append( '' )
                s.append(r['text'])
                s.append( '' )
#                s.append( '-'*10 )
#                s.append( '' )
            elif r['objtype'] in ('def', 'rdef', 'cdef', ):
                # TODO: apply rules to suppress reporting under certain circumstances
                macros.append(r)
                s.append( '' )
                s.append( '.. %s %s %s %d %d' % (self.filename, 
                                              r['objtype'], 
                                              r['name'], 
                                              r['start_line'], 
                                              r['end_line']) )
                s.append( '.. spec:%s:: %s' % ( r['objtype'], r['name'],) )
                s.append('')
                s.append(' '*4 + '*' + r['objtype'] + ' macro declaration*')
                desc = r.get('description', '')
                if len(desc) > 0:
                    s.append('')
                    for line in desc.splitlines():
                        s.append(' '*4 + line)
                s.append( '' )
            elif r['objtype'] in ('function def', 'function rdef',):
                # TODO: apply rules to suppress reporting under certain circumstances
                functions.append(r)
                objtype = r['objtype'].split()[1]
                s.append( '' )
                s.append( '.. %s %s %s %d %d' % (self.filename, 
                                              objtype, 
                                              r['name'], 
                                              r['start_line'], 
                                              r['end_line']) )
                s.append( '.. spec:%s:: %s(%s)' % ( objtype, r['name'], r['args']) )
                s.append('')
                s.append(' '*4 + '*' + r['objtype'].split()[1] + '() macro function declaration*')
                desc = r.get('description', '')
                if len(desc) > 0:
                    s.append('')
                    for line in desc.splitlines():
                        s.append(' '*4 + line)
                s.append( '' )
            
            # Why document local variables in a global scope?
            elif r['objtype'] in ('global', 'constant'):
                # TODO: apply rules to suppress reporting under certain circumstances
                declarations.append(r)
                if r.get('parent') is None:
                    s.append( '.. spec:%s:: %s' % ( r['objtype'], r['name']) )
                    s.append('')
                    if r['objtype'] in ('constant'):
                        s.append(' '*4 + '*constant declaration*')
                    else:
                        s.append(' '*4 + '*' + r['objtype'] + ' variable declaration*')
                    desc = r.get('description', '')
                    if len(desc) > 0:
                        s.append('')
                        for line in desc.splitlines():
                            s.append(' '*4 + line)
                    s.append( '' )

#        s.append( '-'*10 )
#        s.append( '' )

        s += _report_table('Variable Declarations (%s)' % self.filename, declarations, 
                          ('objtype', 'name', 'start_line', 'summary', ))
        s += _report_table('Macro Declarations (%s)' % self.filename, macros, 
                          ('objtype', 'name', 'start_line', 'end_line', 'summary', ))
        s += _report_table('Function Macro Declarations (%s)' % self.filename, functions, 
                          ('objtype', 'name', 'start_line', 'end_line', 'args', 'summary', ))
        #s += _report_table('Findings from .mac File', self.findings, ('start_line', 'objtype', 'line', 'summary', ))

        return '\n'.join(s)

    def ReST(self, style = 'simple'):
        """create the ReStructured Text from what has been found"""
    
        # allow for additional renderers, selectable by options
        renderer_dict =  {'simple': self._simple_ReST_renderer,}
        if style not in renderer_dict:
            raise RuntimeWarning, "%s renderer not found, using `simple`" % style
        return renderer_dict[style]()


def _report_table(title, itemlist, col_keys = ('objtype', 'start_line', 'end_line', )):
    """ 
    return the itemlist as a reST table
    
    :param str title:  section heading above the table
    :param {str,str} itemlist: database (keyed dictionary) to use for table
    :param [str] col_keys: column labels (must be keys in the dictionary)
    :returns [str]: the table (where each list item is a string of reST)
    """
    if len(itemlist) == 0:
        return []
    rows = []
    last_line = None
    for d in itemlist:
        if d['start_line'] != last_line:
            rowdata = [str(d.get(key,'')).strip() for key in col_keys]
            rows.append( tuple(rowdata) )
        last_line = d['start_line']
    return make_rest_table(title, col_keys, rows, '=')


def make_rest_table(title, labels, rows, titlechar = '='):
    """
    build a reST table
    
    :param str title: placed in a section heading above the table
    :param [str] labels: columns labels
    :param [[str]] rows: 2-D grid of data, len(labels) == len(data[i]) for all i
    :param str titlechar: character to use when underlining title as reST section heading
    :returns [str]: each list item is reST
    """
    # could use spec2nexus.rest_table instead, adds a requirement
    #
    # this is commented out since it causes a warning when building:
    #  specmacrofileparser.py:docstring of sphinxcontrib.specmacrofileparser.make_rest_table:14: WARNING: Block quote ends without a blank line; unexpected unindent.
    # -----
    #    """
    #    build a reST table
    #        
    #    :param str title: placed in a section heading above the table
    #    :param [str] labels: columns labels
    #    :param [[str]] rows: 2-D grid of data, len(labels) == len(data[i]) for all i
    #    :param str titlechar: character to use when underlining title as reST section heading
    #    :returns [str]: each list item is reST
    #
    #    Example::
    #        
    #        title = 'This is a reST table'
    #        labels = ('name', 'phone', 'email')
    #        rows = [
    #                ['Snoopy',           '12345', 'dog@house'],
    #                ['Red Baron',        '65432', 'fokker@triplane'],
    #                ['Charlie Brown',    '12345', 'main@house'],
    #        ]
    #        print '\n'.join(make_rest_table(title, labels, rows, titlechar='~'))
    #
    #    This results in this reST::
    #    
    #        This is a reST table
    #        ~~~~~~~~~~~~~~~~~~~~
    #        
    #        ============= ===== ===============
    #        name          phone email          
    #        ============= ===== ===============
    #        Snoopy        12345 dog@house      
    #        Red Baron     65432 fokker@triplane
    #        Charlie Brown 12345 main@house     
    #        ============= ===== ===============
    #    
    #    """
    s = []
    if len(rows) == 0:
        return s
    if len(labels) > 0:
        columns = zip(labels, *rows)
    else:
        columns = zip(*rows)
    widths = [max([len(item) for item in row]) for row in columns]
    separator = " ".join( ['='*key for key in widths] )
    fmt = " ".join( '%%-%ds' % key for key in widths )
    s.append( '' )
    s.append( title )
    s.append( titlechar*len(title) )
    s.append( '' )
    s.append( separator )
    if len(labels) > 0:
        s.append( fmt % labels )
        s.append( separator )
    s.extend( [fmt % tuple(row) for row in rows] )
    s.append( separator )
    return s


TEST_DIR = os.path.join('..', 'macros')


if __name__ == '__main__':
    filelist = [f for f in sorted(os.listdir(TEST_DIR)) if f.endswith('.mac')]
    for item in filelist:
        filename = os.path.join(TEST_DIR, item)
        print filename
        p = SpecMacrofileParser(filename)
        print p.ReST()
        #pprint (p.findings)



'''
Created on Jul 15, 2012

@author: Pete
'''


import os
import re


string_start                = r'^'
string_end                  = r'$'
match_all                   = r'.*'
non_greedy_filler           = match_all + r'?'
non_greedy_whitespace       = r'\s*?'
extended_comment_marker     = r'\"{3}'
extended_comment_match      = r'(' + extended_comment_marker + r')'
macro_name                  = r'[a-zA-Z_][\w_]*'
macro_name_match            = r'(' + macro_name + r')'
arglist_match               = r'(' + match_all + r')'
non_greedy_filler_match     = r'(' + non_greedy_filler + r')'
variable_name_match         = r'(@?' + macro_name + r'\[?\]?)'


spec_macro_declaration_match_re = re.compile(
                        string_start
                        + r'\s*?'                           # optional blank space
                        + r'(r?def)\s'                      # 1: def_type (rdef | def)
                        + non_greedy_whitespace
                        + macro_name_match                  # 2: macro_name
                        + non_greedy_filler_match           # 3: optional arguments
                        + r'\'\{?'                          # start body section
                        + non_greedy_filler_match           # 4: body
                        + r'\}?\''                          # end body section
                        + r'(#.*?)?'                        # 5: optional comment
                        + string_end, 
                        re.IGNORECASE|re.DOTALL|re.MULTILINE)


TESTFILE = os.path.join('..', 'comparison', 'problem-auto.mac')
buf = open(TESTFILE, 'r').read()

for mo in spec_macro_declaration_match_re.finditer(buf):
    print mo.groups(), mo.start(), mo.end(), mo.span()




'''
Created on Jul 11, 2012

@author: Pete
'''


import re
import pyparsing


string_start                = r'^'
string_end                  = r'$'
match_all                   = r'.*'
non_greedy_filler           = match_all + r'?'
non_greedy_whitespace       = r'\s*?'

spec_cdef_declaration_match_re = re.compile( 
            r''
            + non_greedy_filler         # optional any kind of preceding stuff, was \s*? (optional blank space) 
            + non_greedy_whitespace
            + r'(cdef)'                 # 0: cdef
            + r'\('                     # opening parenthesis 
#                + r'(.*?)'                 # 1: args (anything between the parentheses) 
            + r'(.*?)'                   # 1: args (anything between the parentheses) 
            + r'\)',                    # closing parenthesis
            re.DOTALL | re.MULTILINE | re.VERBOSE
         )


barrage = """
cdef("test1")
cdef("test2","\nwait(1);\n","waitmove_hack","0x20")
cdef("test3",
     "\nwait(1);\n",
     "waitmove_hack",
     "0x20")
cdef("test4", "", "scan_cleanup", "delete")
cdef("test5", sprintf("dscan_cleanup $1 %s;", _c1), "dscan")
"""


for mo in spec_cdef_declaration_match_re.finditer(barrage):
    print mo.groups(), mo.start(), mo.end(), mo.span()

print """
It is not possible to identify properly all these cases with a regular expression

Instead, search for the beginnings of a potential cdef declaration 
'cdef\(' and start watching the code, incrementing and decrementing 
on ( and ) until the counter gets back to zero.
"""


#!/usr/bin/env python


'''
$Id: tester3.py 979 2012-06-26 05:14:24Z jemian $

test various regular expressions to match macro declaration signatures
'''


import re

# http://www.greenend.org.uk/rjk/tech/regexp.html

spec_func_sig_re = re.compile(
    r'''^ ([a-zA-Z_]\w*)         # macro name
          ((\s+\S+)*)            # optional: arguments
          $                      # and nothing more
          ''', re.VERBOSE)

test_group = """
example_runtime_defined_macro content
test_macro2 arg1   1.0 2     3
simple_macro
_do_this    4    5    7
5testmacro
"""

for phrase in test_group.split("\n"):
    print "testing:", phrase, "\t --> ",
    m = spec_func_sig_re.match(phrase)
    if m is None:
        print "no match"
    else:
        name, args, last = m.groups()
        print name, args.strip().split()



'''
Created on Jul 6, 2012

@author: Pete

Correct the parser's match to macro function declarations
'''

import re


barrage = '''
def uascanFindFactor(start center finish numPts exponent minStep) '{                                                                                            
def _usaxs_triangulate (rot,center,dist) '{                                                                                                                     
def uascanStepFunc(x, factor, center, exponent, minStep) '{                                                                                                     
'''

spec_function_declaration_match_re = re.compile(
                              r'^'                      # line start
                            + r'\s*?'                   # optional blank space
                            + r'(r?def)'                # 0: def_type (rdef | def)
                            + r'\s*?'                   # optional blank space
                            + r'([a-zA-Z_][\w_]*)'      # 1: function_name
                            + r'\s*?'                   # optional blank space
                            + r'\('                     # opening parenthesis
                            + r'(.*?)'                  # 2: args (anything between the parentheses)
                            + r'\)'                     # closing parenthesis
                            + r'\s*?'                   # optional blank space
                            + r'\''                     # open macro content
                            + r'(.*?)'                  # 3: content, optional
                            + r'(#.*?)?'                # 4: optional comment
                            + r'$'                      # line end
                        )

if __name__ == '__main__':
    for line in barrage.split('\n'):
        m = spec_function_declaration_match_re.match(line)
        if m is not None:
            print m.groups()


# -*- coding: utf-8 -*-

from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

short_desc = 'Sphinx "specdomain" extension'
long_desc = short_desc + '\n'*2 + open('README').read()

version = open('VERSION').read().strip()

requires = ['Sphinx>=1.1.1']

# classifiers: http://pypi.python.org/pypi?%3Aaction=list_classifiers

setup(
    name='sphinxcontrib-specdomain',
    version=version,
    url='http://specdomain.readthedocs.org', 
    license='Argonne OPEN SOURCE LICENSE, see LICENSE file for details',
    author='Pete Jemian',
    author_email='jemian@anl.gov',
    description=short_desc,
    long_description=long_desc,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Science/Research',
        'License :: Free To Use But Restricted',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Documentation',
        'Topic :: Utilities',
    ],
    platforms='any',
    packages=find_packages(exclude=['comparison', 'macros', 'test', 'doc', 'markup_example', ]),
    include_package_data=True,
    install_requires=requires,
    namespace_packages=['sphinxcontrib'],
)



#!/usr/bin/env python

#  $Id: tester.py 978 2012-06-26 03:12:12Z jemian $


'''test various regular expressions to match macro declarations'''


import re

#  http://regexpal.com/
#  http://www.txt2re.com/index-python.php3

def tester(test_cases, regexp):
    for s in test_cases.split('\n'):
        if len(s) > 0:
            print s, ":",
            m = regexp.match(s)
            if m is None:
                print ""
            else:
                parselist = m.groups()
                print parselist
    print "#" + "-"*59

##################################################################################

re_spec_global_declaration = re.compile(
    r'''
    ^                                  # start of line
    (\s*)                              # optional preceding white space
    global                             # "global" declaration
            #
            # FIXME: only finds the last symbol
    ([\s]+[@]?[a-zA-Z_][\w]*(\[\])?)+  # variable name(s) + optional array braces
            #
    (\s+\#.*)*                         # optional comments
    $                                  # end of line
    ''', re.VERBOSE
)

test_cases = '''
global  BCDA_GM[]

   global    theta[]
   global    2theta[]  # this will not be found
   global    _motor[]

global kohzu_PV kohzuMV_PV UND_PV Und_Off UNDE_TRACK_ON
global       kohzuStop_PV kohzuMode_PV      kohzuMove_PV
    global CCD_OVERHEAD_SECS_MEASURED   # measured readout time

    global @A_name[] @B_name[]
       unglobal @A_name
       unglobal @B_name
'''

tester(test_cases, re_spec_global_declaration)

##################################################################################

py_sig_re = re.compile(
    r'''^ ([\w.]*\.)?            # class name(s)
          (\w+)  \s*             # thing name
          (?: \((.*)\)           # optional: arguments
           (?:\s* -> \s* (.*))?  #           return annotation
          )? $                   # and nothing more
          ''', re.VERBOSE)

test_cases = '''
the.parent.class.myFunc(a, b) -> answer
f2()
f4(1, None, "four", four, 4, 4.0, "4.0")
'''

tester(test_cases, py_sig_re)

##################################################################################


match_all = '.*'
non_greedy_filler = match_all+'?'
double_quote_string = '("'+non_greedy_filler+'")'
word_match = '((?:[a-z_][\w]*))'
cdef_match = '(cdef)'

regexp_str = ''
regexp_str += non_greedy_filler + double_quote_string
regexp_str += non_greedy_filler + word_match+'?'
regexp_str += non_greedy_filler + word_match+'?'
regexp_str += non_greedy_filler + word_match+'?'

spec_func_re = re.compile(regexp_str, re.IGNORECASE|re.DOTALL)

test_cases = '''
cdef("macro_name", "commands", "partname", "delete")
cdef("macro_name", "commands", "partname", flags)
cdef("cleanup_once", sprintf("dscan_cleanup $1 %s;", _c1), "dscan")
cdef("macro_name", "commands", "partname")
cdef("macro_name")
thing(arg1,arg2)
afunc("tires")
Afunc()
aFunc(parm1)
cdef("geo_ub_default", "", "ub.mac")
cdef("config_mac", "{PLOT_CNTRS_MAX = COUNTERS}", "PLOT_Y", 0x10 )
'''
buf = open('../macros/cdef-examples.mac', 'r').read()
test_cases += buf
tester(test_cases, spec_func_re)

cdef1_re = re.compile(cdef_match+'\('
                      + '('+match_all+')' 
                      + '\)', 
                      re.IGNORECASE|re.DOTALL)
#cdef2_re = re.compile(cdef_match+'\('
#                      + non_greedy_filler + word_match
#                      + non_greedy_filler + word_match
#                      +'\)', 
#                      re.IGNORECASE|re.DOTALL)
func_start_re = re.compile(word_match+'\(',
                      re.IGNORECASE|re.DOTALL)
func_match_re = re.compile(word_match+'\('
                      + '('+match_all+')' 
                      + '\)', 
                      re.IGNORECASE|re.DOTALL)
# cdef1_re is most general for recognizing any of the different cdef signatures
# func_match_re is most general for recognizing any of the different cdef signatures
tester(test_cases, func_match_re)


##################################################################################

txt='\'"macro_name", "content", "groupname", flags\''

uninteresting_csv = non_greedy_filler+','
csv_match = '(' + non_greedy_filler + '),'

re_str=non_greedy_filler        # Non-greedy match on filler
re_str += double_quote_string   # Double Quote String 1
re_str += non_greedy_filler     # Non-greedy match on filler
re_str += uninteresting_csv     # Uninteresting: csv
re_str += non_greedy_filler     # Non-greedy match on filler
re_str += csv_match             # Command Seperated Values 1
re_str += non_greedy_filler     # Non-greedy match on filler
re_str += uninteresting_csv     # Uninteresting: csv
re_str += non_greedy_filler     # Non-greedy match on filler
re_str += csv_match             # Command Seperated Values 2

re_str = "(.*)"

rg = re.compile(re_str, re.IGNORECASE|re.DOTALL)

test_cases = '''
'"macro_name", "commands", "partname", "delete"'
'"macro_name", "commands", "partname", flags'
'"cleanup_once", sprintf("dscan_cleanup $1 %s;", _c1), "dscan"'
'"macro_name", "commands", "partname"'
'"macro_name"'
'"geo_ub_default", "", "ub.mac"'
'"config_mac", "{PLOT_CNTRS_MAX = COUNTERS}", "PLOT_Y", 0x10'
'''
print re_str
tester(test_cases, rg)
